@@ -70,7 +70,7 @@ CONFIG_SECTION_ORDER = ["paths", "variables", "system", "nlp", "components", "co
 
 logger = logging.getLogger("spacy")
 logger_stream_handler = logging.StreamHandler()
-logger_stream_handler.setFormatter(logging.Formatter("%(message)s"))
+logger_stream_handler.setFormatter(logging.Formatter("[%(asctime)s] [%(levelname)s] %(message)s"))
 logger.addHandler(logger_stream_handler)
 
 
@@ -803,7 +803,7 @@ def run_command(
     stdin (Optional[Any]): stdin to read from or None.
     capture (bool): Whether to capture the output and errors. If False,
         the stdout and stderr will not be redirected, and if there's an error,
-        sys.exit will be called with the returncode. You should use capture=False
+        sys.exit will be called with the return code. You should use capture=False
         when you want to turn over execution to the command, and capture=True
         when you want to run the command more like a function.
     RETURNS (Optional[CompletedProcess]): The process object.
@@ -930,6 +930,8 @@ def is_same_func(func1: Callable, func2: Callable) -> bool:
     """
     if not callable(func1) or not callable(func2):
         return False
+    if not hasattr(func1, "__qualname__") or not hasattr(func2, "__qualname__"):
+        return False
     same_name = func1.__qualname__ == func2.__qualname__
     same_file = inspect.getfile(func1) == inspect.getfile(func2)
     same_code = inspect.getsourcelines(func1) == inspect.getsourcelines(func2)
@@ -1452,9 +1454,10 @@ def is_cython_func(func: Callable) -> bool:
     if hasattr(func, attr):  # function or class instance
         return True
     # https://stackoverflow.com/a/55767059
-    if hasattr(func, "__qualname__") and hasattr(func, "__module__"):  # method
-        cls_func = vars(sys.modules[func.__module__])[func.__qualname__.split(".")[0]]
-        return hasattr(cls_func, attr)
+    if hasattr(func, "__qualname__") and hasattr(func, "__module__") \
+        and func.__module__ in sys.modules:  # method
+            cls_func = vars(sys.modules[func.__module__])[func.__qualname__.split(".")[0]]
+            return hasattr(cls_func, attr)
     return False
 
 
@@ -1497,3 +1500,15 @@ def raise_error(proc_name, proc, docs, e):
 
 def ignore_error(proc_name, proc, docs, e):
     pass
+
+
+def warn_if_jupyter_cupy():
+    """Warn about require_gpu if a jupyter notebook + cupy + mismatched
+    contextvars vs. thread ops are detected
+    """
+    if is_in_jupyter():
+        from thinc.backends.cupy_ops import CupyOps
+        if CupyOps.xp is not None:
+            from thinc.backends import contextvars_eq_thread_ops
+            if not contextvars_eq_thread_ops():
+                warnings.warn(Warnings.W111)
